library(dplyr)
library(openxlsx)
library(plyr)
library(stringi)
args = commandArgs(trailingOnly=TRUE)
filepath <- args[1]

  "Get Indecies of each Selection"
  AggIndecies <- c(3,4,5,6)
  ConcIndecies <- c(1 , 9 ,10 ,11 , 12 , 13)
  AggRoundIndecies <- 10

  MatchedList <<- list(AggIndecies , ConcIndecies  , AggRoundIndecies)
  AllSubReportContent <- NULL
  AllSummaryContent <- NULL
  "Start Processing of data"
  for (subdir in list.dirs(path = filepath, full.names = TRUE,recursive=FALSE)){
    #ReportContent <- NULL
    for(file in list.files(subdir)){
      ReportsPath <- paste0(subdir , '\\' , file)
      ReportContent <- read.xlsx(ReportsPath , sheet = 15)
      ReportContent <- ReportContent[,1:10]
      SortedReportContent <- ReportContent[order(ReportContent$Accession),]

      ProteinCount <- dim(ReportContent)[1]
      #Remove Reversed Proteins
      SortedReportContent <- SortedReportContent[!grepl("REVERSED" , SortedReportContent$Name),]
      RemainingProteins <- dim(SortedReportContent)[1]
      ReversedProteins <- dim(ReportContent)[1] - dim(SortedReportContent)[1]
      ProteinGroups <- max(SortedReportContent$N)
      #Get Names of proteins
      SortedSplit <-  data.frame(do.call('rbind', strsplit(as.character(SortedReportContent$Name),'OS',fixed=TRUE)))
      SortedReportContent <- cbind(SortedReportContent , SortedSplit$X1)
      colnames(SortedReportContent)[11] <- "Names"
      #Remove Duplicates per group
      DistincitReportContent <- SortedReportContent[!duplicated(SortedReportContent[c(1,11)]),]

      #Protein group count per each fraction
      ProteinGroupCount<- ddply(DistincitReportContent, .(DistincitReportContent$N), nrow)
      colnames(ProteinGroupCount) <- c("Protein Group#" , "Number of proteins")
    
            #Get species per group
      OrganismOS <- data.frame(str_match(DistincitReportContent$Name, "OS=(.*?)OX"))
      OrganismOSdup <- data.frame(str_match(SortedReportContent$Name, "OS=(.*?)OX"))
      
      DistincitReportContent <- cbind(DistincitReportContent , as.character(OrganismOS$X2))
      colnames(DistincitReportContent)[12] <- "Specieslist"
      #Species Analysis '
      Speciesframe <- data.frame(cbind(SortedReportContent$N , as.character(OrganismOSdup$X2)))
      Speciesframe <- Speciesframe[!duplicated(Speciesframe[c(1,2)]),]
      Speciesframe <- aggregate(Speciesframe[,2] , by = list(Speciesframe[,1]) , paste,collapse = "||")
      colnames(Speciesframe) <- c("Protein Group" , "Species list")
      #Species Analysis by name 
      #Species Analysis '
      SpeciesframeName <- data.frame(cbind(as.character(SortedReportContent$Names) , as.character(OrganismOSdup$X2)))
      SpeciesframeName <- SpeciesframeName[!duplicated(SpeciesframeName[c(1,2)]),]
      SpeciesframeName <- aggregate(SpeciesframeName[,2] , by = list(SpeciesframeName[,1]) , paste,collapse = "||")
      colnames(SpeciesframeName) <- c("Protein Name" , "Species list")
      #Distinct OS
      DistinctOrganismOS <- unique(OrganismOS$X2)
      NumberofUniqueOrganisms <- length(DistinctOrganismOS)
      #Export data 
      GroupsWorkbook <- createWorkbook()
      addWorksheet(GroupsWorkbook , "Protein Groups")
      addWorksheet(GroupsWorkbook , "Unique Species")
      addWorksheet(GroupsWorkbook , "Whole Species list")
      addWorksheet(GroupsWorkbook , "Protein Species per group")
      addWorksheet(GroupsWorkbook , "Protein Species per Name")
      writeData(GroupsWorkbook, sheet = "Protein Groups", x = ProteinGroupCount)
      writeData(GroupsWorkbook, sheet = "Unique Species", x = OrganismOS$X2)
      writeData(GroupsWorkbook, sheet = "Whole Species list", x = OrganismOSdup$X2)
      writeData(GroupsWorkbook, sheet = "Protein Species per group", x = Speciesframe)
      writeData(GroupsWorkbook, sheet = "Protein Species per Name", x = SpeciesframeName)
      SummaryName <- paste0(subdir , "/" , strsplit(file , ".xlsx")[[1]]," Summary.xlsx")
      saveWorkbook(GroupsWorkbook, SummaryName)
      
      #Summary
      DuplicatesPergroupCount <- dim(SortedReportContent)[1] - dim(DistincitReportContent)[1]
      Summarylist <- c(file , ProteinCount , ProteinGroups , ReversedProteins , RemainingProteins  , DuplicatesPergroupCount , NumberofUniqueOrganisms)
      AllSummaryContent <- cbind(AllSummaryContent , Summarylist)
      FractionName <- rep(file , dim(DistincitReportContent)[1])
      DistincitReportContent <- cbind(DistincitReportContent , FractionName)
      AllSubReportContent <- rbind(AllSubReportContent,DistincitReportContent)
    }
    AllSortedReportContent <- AllSubReportContent[order(AllSubReportContent$Accession),]
    AvgCOlnames <- colnames(AllSortedReportContent)[MatchedList[[1]]]
    ConcColnames <- colnames(AllSortedReportContent)[MatchedList[[2]]]
    AvgRoundedColnames <- colnames(AllSortedReportContent)[MatchedList[[3]]]

    AvgeragedReportContent <- aggregate(AllSortedReportContent[,AvgCOlnames], list(MainAccesion =AllSortedReportContent[,7]), FUN = mean)
    AvgeragedRoundedReportContent <- aggregate(AllSortedReportContent[,AvgRoundedColnames], list(MainAccesion =AllSortedReportContent[,7]) , FUN = mean)
    AvgeragedRoundedReportContent[,-1] <-ceiling(AvgeragedRoundedReportContent[,-1])
    ConcReportContent <- setNames(aggregate(AllSortedReportContent[,ConcColnames],
                                            by = list(MainAccesion=AllSortedReportContent[,7]),
                                            paste, collapse="||"),c(colnames(AllSortedReportContent)[7],ConcColnames))

    ConcReportContent[ConcReportContent == "$"] <- " "
    ConcReportContent[ConcReportContent == "NA"] <- " "
    ConcReportContent[ConcReportContent == "$NA"] <- " "
    ConcReportContent[ConcReportContent == "NA$"] <- " "

    colnames(AvgeragedRoundedReportContent)[2] <- "Peptides"
    PreReportContent <- merge(AvgeragedReportContent , AvgeragedRoundedReportContent , by = 1)
    FinalReportContent <- merge(PreReportContent , ConcReportContent , by = 1)
     FinalReportContent<-FinalReportContent %>%
      mutate_all(as.character)

    colnames(AllSummaryContent) <- AllSummaryContent[1,]
    AllSummaryContent <- AllSummaryContent[-1,]
    print(AllSummaryContent)

    rownames(AllSummaryContent) <- c("Proteins count" ,"Protein Groups" , "Reversed proteins" , "Remaining proteins"  , "Number of duplicated proteins" , "Number of Unique species")
    AllSummaryContent <- as.data.frame(AllSummaryContent)
    OutputSplitted <- strsplit(subdir , '/')
    OutputName <- OutputSplitted[[1]][length(OutputSplitted[[1]])]
    OutputPath <- paste0(subdir , '\\' , paste0(OutputName , '.xlsx'))
    SummaryName <- paste0(subdir , '\\' ,  'Summary.csv')
    #Species Analysis 
    UniqueNames <- FinalReportContent$Names
    UniqueNamesSplit <- data.frame(do.call('rbind', strsplit(as.character(FinalReportContent$Name),'||',fixed=TRUE)))
    NamesSpecies <- data.frame(cbind(as.character(UniqueNamesSplit$X1) , as.character(FinalReportContent$Specieslist)))
    SpeciesAnalysis <- aggregate(NamesSpecies[,2],
                                 by = list(Names=NamesSpecies[,1]),
                                 paste, collapse="||")
    colnames(SpeciesAnalysis) <- c("Protein Name","Species list")
    
    FinalWorkbook <- createWorkbook()
    addWorksheet(FinalWorkbook , "Protein Species")
    addWorksheet(FinalWorkbook , "Combined")
    writeData(FinalWorkbook, sheet = "Protein Species", x = SpeciesAnalysis)
    writeData(FinalWorkbook, sheet = "Combined", x = FinalReportContent)
    
    saveWorkbook(FinalWorkbook, OutputPath)
    write.csv(AllSummaryContent, SummaryName)

    ReportContent <- NULL
    AllSubReportContent <- NULL
    AllSummaryContent <- NULL
  }